/**
 * Object that creates jQuery element hiearchy after attaching to DOM and
 * calls 'create_struct' callback when finished. Define your element
 * hiearchy under protected property 'els' and fetch them using 'get_el'
 * method of domel.
 *
 * @inherits domel, caller
 */
pwf.rc('jq.struct', {
	'parents':['jq.domel', 'caller'],

	'proto':{
		/** @param Array els Hiearchy of elements */
		'els':[],

		/** @param null|string prefix Prefix element css classes with this string */
		'prefix':null,


		/**
		 * Action to call after object is attached to DOM. Calls protected
		 * method create_struct after creating elements.
		 *
		 * @return void
		 */
		'el_attached':function(p)
		{
			p('create_els');
			p('create_struct');
		},


		/**
		 * Create element hiearchy.
		 *
		 * @return void
		 */
		'create_els':function(p, els)
		{
			var
				els = typeof els == 'undefined' ? p('els'):els,
				par = this.get_el();

			for (var i = 0, len = els.length; i < len; i++) {
				p('create_el', par, els[i]);
			}
		},


		/**
		 * Create element and bind it in object hiearchy.
		 *
		 * @param jQuery        par  Parent of this element
		 * @param string|Object data Data to be passed
		 * 	{
		 * 		'els':["children", "elements"],
		 * 		'global':true,
		 * 		'name':"name_and_class_of_the_element",
		 * 		'prefix':"override-object-prefix",
		 * 		'tag':"element_tag_default_div",
		 * 	}
		 *
		 * @return jQuery
		 */
		'create_el':function(p, par, data)
		{
			var
				pref = p('prefix'),
				el;

			// Accept strings in array as names
			if (typeof data == 'string') {
				data = {'name':data};
			} else {
				/*
				 * Clone the object to prevent overwriting structure and potential
				 * breaking of other struct classes
				 */
				data = pwf.merge(data);
			}

			// Change tagname to default if not given
			if (!('tag' in data)) {
				data.tag = 'div';
			}

			// Add default CSS class prefix if not given
			if (!('prefix' in data)) {
				data.prefix = pref;
			}

			if ('ui' in data) {
				// Expect domel object
				par[data.name] = pwf.resolve(data.ui, pwf.merge(data, {
					'parent':par
				}));

				el = par[data.name].get_el().addClass((data.prefix ? data.prefix + '-':'') + data.name);
			} else {
				// Create element and reference it with its' name under its' parent
				par.create_els([data.name], data.tag, data.prefix);
				el = par[data.name];
			}

			// Insert locale string if available
			if (pwf.status('locales') && typeof data.trans != 'undefined') {
				var msg = data.trans;

				if (msg instanceof Object) {
					el.html(pwf.locales.trans(msg.str, msg.params));
				} else {
					el.html(pwf.locales.trans(msg));
				}
			}

			if ('html' in data) {
				el.html(data.html);
			}

			// Setup elements' initial CSS
			if ('css' in data) {
				el.css(data.css);
			}

			if (data.attrs instanceof Object) {
				for (var key in data.attrs) {
					el.attr(key, data.attrs[key]);
				}
			}

			// Check for cname key and add CSS classes to created element.
			if (data.cname instanceof Array) {
				for (var i = 0; i < data.cname.length; i++) {
					el.addClass(data.cname[i]);
				}
			}

			// Create hiearchy under this element if given
			if ('els' in data) {
				for (var i = 0, len = data.els.length; i < len; i++) {
					p('create_el', el, data.els[i]);
				}
			}

			// Register this element as global - accessible via #get_el(name)
			if ('global' in data && data.global) {
				this.get_el()[data.name] = el;
			}

			return el;
		}
	}
});
